#ifndef C_CUT_H
#define C_CUT_H

#include "Csubsets.h"
#include <vector>

typedef int_vector_type betti_container_type;
enum print_state_type{CUT,REALIZATION,BETTI,RANK,SOL2,ALL};



//declaration of class Ccut
//------------------------------------------------------
class Ccut : public Csubsets
{
	public:
		Ccut() : Csubsets(), _realizable(false), realization(0), short_elements(0),//
				print_state(CUT), betti_numbers(0), _rank(0) {}
		explicit Ccut(const Csubsets& c) : Csubsets(c), _realizable(false), realization(0),//
				short_elements(0), print_state(CUT), betti_numbers(0), _rank(0) {}
		Ccut(const Ccut& c) : Csubsets((Csubsets)c), _realizable(c._realizable),//
				realization(c.realization), short_elements(c.short_elements), print_state(c.print_state),//
				betti_numbers(c.betti_numbers), _rank(0) {}
		~Ccut() {}

		bool realize();
		void vertices();
		void betti();
		void rank();
		void sol2();

		pos_int_type ns(pos_int_type i) const;

		bool realizable() const {return _realizable;}
		const real_vector_type& get_realization() const {return realization;}
		const container_type& get_short_elements() const;
		
		bool contains(const Csubset& J) const;
		bool contains(pos_int_type i) const {return contains(Csubset(i));}
		
		void clear();
		Ccut& print(print_state_type ps);

				
	private:
		bool _realizable;
		real_vector_type realization;
		mutable container_type short_elements;
		print_state_type print_state;
		betti_container_type betti_numbers;
		pos_int_type _rank;
		pos_int_type _sol2;

		void prepare_realization();
		void calculate_short_elements() const;
		
		void print_cut(ostream& os) const;
		void print_realization(ostream& os) const;
		void print_betti(ostream& os) const;
		void print_rank(ostream& os) const;
		void print_sol2(ostream& os) const;
	
		
	friend ostream& operator<<(ostream&, const Ccut&);
	friend class Ccuts;
};
//------------------------------------------------------



//operator overloading for printing
//------------------------------------------------------
ostream& operator<<(ostream&, const Ccut&);
ostream& operator<<(ostream&, const betti_container_type&);
//------------------------------------------------------

#endif
