#include <cstdlib>
#include <stdio.h>
//#include <strstream>
#include <cassert>
#include <algorithm>
//#include <math.h>
#include "Ccut.h"
#include "Cneutral.h"
#include "Ccuts_error.h"
#include "Csubsets.h"
#include "basic_math.h"
#include "linear_programming.h"
#include "basic_output.h"
#include "Csol2.h"



bool Ccut::realize()
{
	prepare_realization();
	
	try
	{
		lp::simplex();
	}
	catch(Clp_error e)
	{
		switch (e.state)
		{
			case OPTIMAL:
				_realizable = true;
				realization = lp::results();
				break;

			case NOT_FEASIBLE:
				_realizable = false;
				break;

			case UNBOUNDED:
				_realizable = false;
				break;
			
			default:
				_realizable = false;
				break;
		}
	}
	
	return _realizable;
}



void Ccut::vertices()
{
/*	prepare_realization();
	
	ostrstream name;
	
	name << RES_VERTICES_DIR << "{" << (Csubsets)(*this) << "}";
	output.file(name.str());
	
	output << "{" << (Csubsets)(*this) << "}" << endl
		<< "H-representation"<< endl
		<< "begin" << endl
		<< lp::A.size() << " " << lp::A[0].size() + 1 << " rational" << endl;
	
	assert(lp::A.size() == lp::b.size());
	
	real_matrix_type::const_iterator i(lp::A.begin());
	real_vector_type::const_iterator ii(lp::b.begin());
	real_vector_type::const_iterator j;
	
	for(;i != lp::A.end(); ++i, ++ii)
	{
		output << *ii << " ";
		for(j = (*i).begin(); j != (*i).end(); ++j)
		{
			output << -(*j) << " ";
		}
		output << endl;
	}
	
	output<< "end" << endl;
*/
}



void Ccut::betti()
{
	if(betti_numbers.empty())
	{
		int_type b(0);
	
		for(pos_int_type i(0); i < Csubset::N - 2; ++i)
		{
			b = ns(i) - ns(Csubset::N - 2 - i) + b;
			betti_numbers.push_back(b);
		}
	}
}



void Ccut::rank()
{
	_rank = 1 + ns(1) - ns(Csubset::N - 3) - ns(Csubset::N - 4);
}
	


void Ccut::sol2()
{
	Csol2 s(*this);
	_sol2 = s.sol2();
}



pos_int_type Ccut::ns(pos_int_type i) const
{
	//output << "Entering ns() with i = " << i << endl;

	calculate_short_elements();
	
	pos_int_type count(0);
	
	//output << "size " << short_elements.size() << endl;
	
	for(const_iterator k(short_elements.begin()); k != short_elements.end(); ++k)
	{
		//output << "element " << *k << endl;
		//output << "element size " <<  k->size() << endl;
		//output << "element contains N " << k->containsN() << endl;
		
		if((k->containsN()) && (k->size() == (i + 1)))
		{
			//output << "element size " <<  k->size() << endl;
			//output << "element contains N " << k->containsN() << endl;
			//output << "good element " << *k << endl;
			//output << "count " << count << endl;
			
			++count;
		}
		else
		{
			//output << "bad element " << *k << endl;
		}
	}
	
	//output << "Leaving ns()" << endl;
	return count;
}



const Ccut::container_type& Ccut::get_short_elements() const
{
	calculate_short_elements();
	return short_elements;
}



bool Ccut::contains(const Csubset& J) const
{
	calculate_short_elements();
	if(find(short_elements.begin(),short_elements.end(),J) == short_elements.end())
	{
		return false;
	}
	
	return true;
}



void Ccut::clear()
{
	Csubsets::clear();
	realization.clear();
	short_elements.clear();
	betti_numbers.clear();
}
		


Ccut& Ccut::print(print_state_type ps)
{
	print_state = ps;
	return *this;
}



//PRIVATE FUNCTIONS ...
void Ccut::prepare_realization()
{
	//we start by calculating the short elements (ie the ones
	//that are short when we imposed this cut)
	calculate_short_elements();
	
	
	//We now initialize the 3 elements needed for our problem: A, b, c
	//Remember we are transforming the elements in S (elements that must be short)
	//in a linear programming problem. We write the conditions under the form
	//max z = cx under conditions: Ax <= b
	
	//we must (I am not sure ...) add also the fact that x(i) <= x(i+1)
	
	lp::A.resize((short_elements.size() + Csubset::N - 1),Csubset::N);
	lp::b.assign((short_elements.size() + Csubset::N - 1),-1);
	
	//un peu magouille ... je remplis le vecteur avec des 0 pour les contraintes
	//du type x(i) <= x(i+1)
	for(pos_int_type i = short_elements.size(); i < lp::b.size(); i++)
	{
		lp::b[i] = 0;
	}
	
	//maybe do this better one day - seperate the three things
	//(1) the matrix of constraints A
	//(2) the vector of constraints b
	//(3) the objective function c
	//also I am not convinced with this lp:: stuff for the
	//variables - maybe better local ?
	
	//version 1 - we try to minimize the perimeter
	//lp::c.assign(Csubset::N,-1);
	
	//version 2 - we try to minimize the first side (the shortest)
	lp::c.assign(Csubset::N,0);
	lp::c[0] = -1;
	
	
	pos_int_type i(0), x(0), y(0);
	int_type j(0);
	
	for(Csubsets::const_iterator k(short_elements.begin()); k != short_elements.end(); k++)
	{
		j = (Csubset::N - 1);
		x = (*k).get_code();
		while(j != -1)
		{
			y = power(2,j);
			if(div(x,y).quot == 0)
			{
				lp::A[i][j] = -1;	
			}
			else
			{
				lp::A[i][j] = 1;
				x = x - y;
			}
			--j;
		}
		++i;
	}
	
	//aussi un peu magouille... c'est pour les 1 et -1
	for(i = short_elements.size(); i < lp::A.size(); ++i)
	{
		for(pos_int_type k(0); k < lp::A[0].size(); ++k)
		{
			if(i == (k + short_elements.size()))
			{
				lp::A[i][k] = 1;
				lp::A[i][k+1] = -1;
				k++;
			}
			else
			{
				lp::A[i][k] = 0;
			}
		}
	}
	
	//Maybe it is better not to impose as constraints all the 2^n-1 elements,
	//but just the maximal ones and the relation x_i <= x_i+1. You should
	//have a smaller matrix.
	//TODO in a later version...
}



void Ccut::calculate_short_elements() const
{
	//output << "Entering calculate_short_elements() ...." << endl;
	if(short_elements.empty())
	{
		//output << "Entering the IF in calculate_short_elements() ...." << endl;
		//First we get all the elements that are formally short.	
		Csubset ii(power(2,Csubset::N));
		for(Csubset i(power(2,Csubset::N - 1)); i != ii; ++i)
		{
			if((!i) < i)
			{
				short_elements.push_back(!i);
			}
		}
	
	
		//We now get all the elements from N (the neutral elements)
		//that are short once you impose that the ones in this cut are.	
		Cneutral N;
		N.initialize(Csubset::N);
	
		bool small;
		for(Cneutral::const_iterator i(N.begin()); i != N.end(); ++i)
		{
			small = false;
			for(Csubsets::const_iterator j(elements.begin()); j != elements.end(); ++j)
			{
				if((*i) < (*j)) {small = true;}
			}

			if(small)
			{
				short_elements.push_back(*i);
				++i;
			}
			else
			{
				++i;
				short_elements.push_back(*i);
			}
		}

		if(short_elements.size() != power(2,(Csubset::N - 1)))
		{
			throw Ccuts_error("The number of short elements is always 2^(n-1).");
		}
	}
	//output << "Leaving calculate_short_elements() ...." << endl;
}		



void Ccut::print_cut(ostream& os) const
{
	os << "<" << (Csubsets)(*this) << ">";	
}



void Ccut::print_realization(ostream& os) const
{
	if(_realizable)
	{
		os << ";(";
		real_vector_type::const_iterator i(realization.begin());
		real_vector_type::const_iterator ii(realization.end()); --ii;
		real_type p(0);
		for(; i != ii; ++i)
		{
			p+=(*i);
			os << *i << " ";
		}
		p+=(*i);
		os << *i << ")";
		os << ";" << p;		
	}
	else
	{
		os << ";NR";
	}
}



void Ccut::print_betti(ostream& os) const
{
	os << ";" << betti_numbers;
}



void Ccut::print_rank(ostream& os) const
{
	os << ";" << _rank;
}



void Ccut::print_sol2(ostream& os) const
{
	os << ";" << _sol2;
}



//FRIEND FUNCTIONS ...
ostream& operator<<(ostream& o, const Ccut& c)
{
	c.print_cut(o);
	switch(c.print_state)
	{
		case REALIZATION:
			c.print_realization(o);
			break;
		
		case BETTI:
			c.print_betti(o);
			break;
		
		case RANK:
			c.print_rank(o);
			break;
			
		case SOL2:
			c.print_sol2(o);
			break;
		
		case ALL:
			c.print_realization(o);
			c.print_betti(o);
			c.print_rank(o);
			c.print_sol2(o);
			break;
		
		default:
			break;
	}
	
	return o;
}
	


ostream& operator<<(ostream& os, const betti_container_type& b)
{
	pos_int_type ii(div(Csubset::N-3,2).quot);
	if(ii > 0)
	{
		--ii;
		pos_int_type i(0);
		for(; i != ii; ++i)
		{
			os << b[i+1] << " ";
		}
		os << b[i+1];
	}
	
	return os;
}



