#include "Csol2.h"
#include "Ccut.h"
#include "basic_output.h"
#include <cassert>
using namespace std;



//class Csol2
//-----------------------------------------------------
Csol2::Csol2(const Ccut& c) : R2(c), R3(c) {}



pos_int_type Csol2::sol2()
{
	if(R3.no_equations) {return 0;}
	
	pos_int_type nb_sol(0);
	pos_int_type max_sol(power(2,R2.nb_var));
			
	for(pos_int_type i(0); i != max_sol; ++i)
	{
		if(valid_solution(i))
		{
			++nb_sol;
		}
	}

	return nb_sol;
}



bool Csol2::valid_solution(pos_int_type i)
{
	Cbin_int b(i,R2.nb_var);
	//bool b2(R2.valid_solution(b));
	//bool b3(R3.valid_solution(b));
	
	/*
	if(b2)
	{
		output << b << " valid for R2" << endl;
		if(b3)
		{
			output << b << " valid for R3" << endl;
		}
		else
		{
			output << b << " not valid for R3" << endl;
		}
	}
	else
	{
		output << b << " not valid for R2" << endl;
	}
	*/
	
	if(R2.valid_solution(b) && R3.valid_solution(b))
	{
		return true;
	}

	return false;
}



//class CR2
//-----------------------------------------------------
CR2::CR2(const Ccut& c) : nb_var(Csubset::N - 1)
{
	pos_int_type p(power(2,Csubset::N - 1));
	
	for(pos_int_type i(0); i != (Csubset::N - 1); ++i)
	{
		Csubset a(p + power(2,i));
		if(c.contains(a))
		{
			for(pos_int_type j(0); j != i; ++j)
			{
				a.code = p + power(2,i) + power(2,j);
				if(!c.contains(a))
				{
					relations.push_back(make_pair(i,j));
				}
			}
		}
		else
		{
			--nb_var;
		}
	}
	/*
	output << endl << endl << c << endl;
	output << "Nombre de variables:" << nb_var << endl;
	if(relations.empty())
	{
		output << "Pas de relations de type R2 de degr 2" << endl;
	}
	else
	{
		output << relations << endl;
	}
	*/
}



bool CR2::valid_solution(const Cbin_int& b)
{	
	for(const_iterator i(begin()); i != end(); ++i)
	{
		if((b[i->first] == 1) && (b[i->second] == 1))
		{
			return false;
		}
	}
	
	return true;
}


		
//class CR3
//-----------------------------------------------------		
CR3::CR3(const Ccut& c) : no_equations(false), relations(0)
{
	pos_int_type p(power(2,Csubset::N-1));
	pos_int_type s(0);
	Csubset a(0);
	for(pos_int_type i(0); i < p; ++i)
	{
		a.code = i;
		s = a.size();
		if((!c.contains(a)) && (s <= 3))
		{
			relations.resize(relations.size() + 1);
			for(pos_int_type j(0); j < Csubset::N-1; ++j)
			{
				if(a.contains(power(2,j)) && c.contains(power(2,j)+p))
				{
					relations.back().push_back(j);
				}
			}

			if(relations.back().empty())
			{
				no_equations = true;
				//output << "Pas d'quations" << endl;
				return;
			}
		}
	}
	/*
	if(relations.size() == 0)
	{
		output << "Pas de relations de type R3" << endl;
	}
	else
	{
		for(const_iterator i(relations.begin()); i!=relations.end(); ++i)
		{
			for(sub_const_iterator j(i->begin()); j != i->end(); ++j)
			{
				output << "V" << (*j) + 1;
				if(j+1 != i->end()) {output << " + ";}
			}
			output << "=1" << endl;
		}
	}
	*/
}



bool CR3::valid_solution(const Cbin_int& b)
{
	if(relations.size() == 0) return true;

	pos_int_type one_count(0);
	
	for(const_iterator i(relations.begin()); i != relations.end(); ++i)
	{
		one_count = 0;
		for(sub_const_iterator j(i->begin()); j != i->end(); ++j)
		{
			if(b[*j] == 1) {++one_count;}
			if(one_count > 1) {return false;}
		}
		if(one_count == 0) {return false;}
	}

	return true;
}



//operators for printing
//-----------------------------------------------------		
ostream& operator<<(ostream& os, const CR2& R2)
{
	os << R2.relations;
	return os;
}



ostream& operator<<(ostream& os, const CR2::container_type& rel)
{
	for(CR2::const_iterator i(rel.begin()); i != rel.end(); ++i)
	{
		os << "V" << (i->first + 1) << "V" << (i->second + 1) << "=0 ";
	}
	
	return os;
}


