#include <cmath>
#include "basic_math.h"



//implementation of template functions
//------------------------------------------------------
template<class T>
Tmatrix<T>::Tmatrix(size_type i, size_type j, const T& v) : M(i,j)
{
	for(iterator ii(M.begin()); ii != M.end(); ii++)
	{
		for(vector<T>::iterator jj((*ii).begin()); jj != (*ii).end(); ++jj)
		{
			(*jj) = v;
		}
	}
}



template<class T>
void Tmatrix<T>::resize(size_type i, size_type j)
{
	if((M.size() != i) || (M[0].size() != j))
	{
		M.resize(i);
		for(iterator k(M.begin()); k != M.end(); ++k)
		{
			(*k).resize(j);
		}
	}
}



template<class T>
void Tmatrix<T>::pivot(size_type pivot_line, size_type pivot_column)
{
	vector<T> pivot_line_elements(M[pivot_line]);
	T pivot_column_element(0);
	T temp(0);

	for(size_type i(0); i < M.size(); ++i)
	{
		if(i == pivot_line)
		{
			for(size_type j(0); j < M[0].size(); ++j)
			{
				if(j == pivot_column)
				{
					temp = 1 / pivot_line_elements[pivot_column];
					if(abs(temp) < EPS) {M[i][j] = 0;}
					else {M[i][j] = temp;}
				}
				else
				{
					temp = -(pivot_line_elements[j] / pivot_line_elements[pivot_column]);
					if(abs(temp) < EPS) {M[i][j] = 0;}
					else {M[i][j] = temp;}
				}
			}
		}
		else
		{
			pivot_column_element = M[i][pivot_column];
			for(size_type j(0); j < M[0].size(); ++j)
			{
				if(j == pivot_column)
				{
					temp = pivot_column_element / pivot_line_elements[pivot_column];
					if(abs(temp) < EPS) {M[i][j] = 0;}
					else {M[i][j] = temp;}
				}
				else
				{
					temp = ((M[i][j] * pivot_line_elements[pivot_column]) - (pivot_column_element * pivot_line_elements[j])) / //
						pivot_line_elements[pivot_column];
					if(abs(temp) < EPS) {M[i][j] = 0;}
					else {M[i][j] = temp;}
				}
			}
		}
	}
}



template<class T>
ostream& operator<<(ostream& o, const Tmatrix<T>& A)
{
	for(typename Tmatrix<T>::size_type i(0); i < A.size(); i++)
	{
		for(typename Tmatrix<T>::size_type j(0); j < A[0].size(); ++j)
		{
			o << A[i][j] << '\t';
		}
		o << endl;
	}
	o << endl;

	return o;
}



template<class T>
ostream& operator<<(ostream& o, const vector<T>& V)
{
	for(typename Tmatrix<T>::size_type i(0); i < V.size(); ++i)
	{
		o << V[i] << endl;
	}
	o << endl;
	return o;
}
